use serde::{Deserialize, Serialize};
use slotmap::SlotMap;
use std::collections::HashMap;

use crate::id_types::*;

#[derive(Debug, Clone, Copy, PartialEq, Eq, Serialize, Deserialize)]
pub enum DataType {
    Vector,
    Scalar,
    Selection,
    Mesh,
    Enum,
    // The path to a (possibly new) file where export contents will be saved to
    NewFile,
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub enum InputParamMetadata {
    MinMaxScalar { min: f32, max: f32 },
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct Param {
    pub id: ID,
    pub kind: Descriptor,
    pub node: NodeId,
    pub draw_connector: bool,
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct Node {
    pub id: NodeId,
    pub op_name: String,
    pub inputs: Vec<(String, ID)>,
    pub outputs: Vec<(String, ID)>,
}

#[derive(Default, Debug, Clone, Serialize, Deserialize)]
pub struct Graph {
    pub nodes: SlotMap<NodeId, Node>,
    pub inputs: SlotMap<ID, Param>,
    pub outputs: SlotMap<ID, Param>,
    // Connects the input of a node, to the output of its predecessor that
    // produces it
    pub connections: HashMap<ID, ID>,
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub enum Descriptor {
    Button,
    Slider { min: i32, max: i32 },
    Info,
}

pub struct NodeDescriptor {
    pub op_name: String,
    pub inputs: Vec<(String, Descriptor)>,
    pub outputs: Vec<(String, Descriptor)>,
}
