use editor_state::GraphEditorState;

mod graph_node_ui;

mod color_hex_utils;
mod device;
mod editor_state;
mod graph_editor_egui;
mod graph_impls;
mod graph_types;
mod id_types;
mod index_impls;
mod node_finder;
mod param_ui;

use device::Device;

#[derive(Default)]
struct App {
    pub state: GraphEditorState,
    pub devices: Vec<Device>,
}

impl App {
    fn new() -> Self {
        Self {
            state: GraphEditorState::default(),
            devices: dbg!(Device::query()),
        }
    }
}

impl epi::App for App {
    fn update(&mut self, ctx: &egui::CtxRef, _frame: &epi::Frame) {
        egui::CentralPanel::default().show(ctx, |_| {
            graph_editor_egui::draw_graph_editor(&ctx, &mut self.state, &self.devices);
        });
    }

    fn name(&self) -> &str {
        "Device Node Mapper"
    }
}

fn main() {
    let options = eframe::NativeOptions {
        ..Default::default()
    };

    eframe::run_native(Box::new(App::new()), options);
}
